from flaskunittest import FlaskTestCase

from datetime import datetime, timedelta
import jobmanagement
from server import query


class JobmanagementTestCase(FlaskTestCase):
	def getJobCount(self, state=None):
		if not state:
			data = query("SELECT count(id) AS count from jobs")
		else:
			data = query("SELECT count(id) AS count FROM jobs WHERE state=?", state)
		return data[0]['count']

	def getCanceledJobCount(self):
		data = query("SELECT count(id) AS count from jobs WHERE canceled=1")
		return data[0]['count']

	def generateTestJob(self):
		return jobmanagement.schedule_job('testjob', data={'data': 'mytestdata'})

	def moveJobScheduletimeToPast(self, id, seconds=500):
		query("UPDATE jobs SET time_scheduled = ? WHERE id = ?", datetime.now() - timedelta(seconds=seconds), id)

	def test_schedule_job(self):
		with self.requestContext:
			jobCountBefore = self.getJobCount()
			self.generateTestJob()
			assert(jobCountBefore + 1  == self.getJobCount())

	
	def test_cancel_job(self):
		with self.requestContext:
			canceledJobCountBefore = self.getCanceledJobCount()
			jobmanagement.cancel_job(self.generateTestJob())
			canceledJobCountAfter = self.getCanceledJobCount()
			assert(canceledJobCountBefore +1  == canceledJobCountAfter)

	def test_catch_broken(self):
		with self.requestContext:
			readyJobCountBefore = self.getJobCount('ready')
			jobid = self.generateTestJob()
			self.moveJobScheduletimeToPast(jobid)
			jobmanagement.job_set_state(jobid, 'scheduled')
			jobmanagement.job_catch_broken()
			readyJobCountAfter = self.getJobCount('ready')
			assert(readyJobCountBefore + 1 == readyJobCountAfter)

	def test_job_set_state(self):
		with self.requestContext:
			jobCountBefore = self.getJobCount('teststate')
			jobid = self.generateTestJob()
			jobmanagement.job_set_state(jobid, 'teststate')
			assert(jobCountBefore + 1  == self.getJobCount('teststate'))
